@@ -754,14 +754,14 @@ set_BINMODE()
 		lintwarn(_("`BINMODE' is a gawk extension"));
 	}
 	if (do_traditional)
-		BINMODE = 0;
+		BINMODE = TEXT_TRANSLATE;
 	else if ((BINMODE_node->var_value->flags & NUMBER) != 0) {
 		BINMODE = (int) force_number(BINMODE_node->var_value);
 		/* Make sure the value is rational. */
-		if (BINMODE < 0)
-			BINMODE = 0;
-		else if (BINMODE > 3)
-			BINMODE = 3;
+		if (BINMODE < TEXT_TRANSLATE)
+			BINMODE = TEXT_TRANSLATE;
+		else if (BINMODE > BINMODE_BOTH)
+			BINMODE = BINMODE_BOTH;
 	}
 	else if ((BINMODE_node->var_value->flags & STRING) != 0) {
 		v = BINMODE_node->var_value;
@@ -783,13 +783,13 @@ set_BINMODE()
 				BINMODE = p[0] - '0';
 				break;
 			case 'r':
-				BINMODE = 1;
+				BINMODE = BINMODE_INPUT;
 				break;
 			case 'w':
-				BINMODE = 2;
+				BINMODE = BINMODE_OUTPUT;
 				break;
 			default:
-				BINMODE = 3;
+				BINMODE = BINMODE_BOTH;
 				goto bad_value;
 				break;
 			}
@@ -797,21 +797,21 @@ set_BINMODE()
 		case 2:
 			switch (p[0]) {
 			case 'r':
-				BINMODE = 3;
+				BINMODE = BINMODE_BOTH;
 				if (p[1] != 'w')
 					goto bad_value;
 				break;
 			case 'w':
-				BINMODE = 3;
+				BINMODE = BINMODE_BOTH;
 				if (p[1] != 'r')
 					goto bad_value;
 				break;
+			}
 			break;
 		default:
 	bad_value:
 			lintwarn(_("BINMODE value `%s' is invalid, treated as 3"), p);
 			break;
-			}
 		}
 	}
 	else
@@ -1138,8 +1138,10 @@ r_get_lhs(NODE *n, int reference)
 		if (n->orig_array->type == Node_var_array)
 			fatal(_("attempt to use array `%s' in a scalar context"),
 					array_vname(n));
-		n->orig_array->type = Node_var;
-		n->orig_array->var_value = Nnull_string;
+		if (n->orig_array->type != Node_var) {
+			n->orig_array->type = Node_var;
+			n->orig_array->var_value = Nnull_string;
+		}
 		/* fall through */
 	case Node_var_new:
 		n->type = Node_var;
@@ -1778,8 +1780,13 @@ top:
 							m->vname);
 				isparam = TRUE;
 				save_symbol = m = GET_PARAM(m->param_cnt);
-				if (m->type == Node_array_ref)
+				if (m->type == Node_array_ref) {
+					if (m->orig_array->type == Node_var) {
+						/* gawk 'func f(x) { a = 10; print x; } BEGIN{ f(a) }' */
+						goto uninitialized_scalar;
+					}
 					m = m->orig_array;
+				}
 			}
 				
 			switch (m->type) {
@@ -1795,6 +1802,7 @@ top:
 				break;
 
 			case Node_var_new:
+uninitialized_scalar:
 				m->type = Node_var;
 				m->var_value = Nnull_string;
 				if (do_lint)
@@ -2160,14 +2168,26 @@ post:
 			t1 = force_string(*lhs);
 			t2 = POP_STRING();
 
-			free_wstr(*lhs);
-
 			if (t1 != t2 && t1->valref == 1 && (t1->flags & PERM) == 0) {
 				size_t nlen = t1->stlen + t2->stlen;
 				erealloc(t1->stptr, char *, nlen + 2, "r_interpret");
 				memcpy(t1->stptr + t1->stlen, t2->stptr, t2->stlen);
 				t1->stlen = nlen;
 				t1->stptr[nlen] = '\0';
+
+#if MBS_SUPPORT
+				if ((t1->flags & WSTRCUR) != 0 && (t2->flags & WSTRCUR) != 0) {
+					size_t wlen = t1->wstlen + t2->wstlen;
+
+					erealloc(t1->wstptr, wchar_t *,
+							sizeof(wchar_t) * (wlen + 2), "r_interpret");
+					memcpy(t1->wstptr + t1->wstlen, t2->wstptr, t2->wstlen);
+					t1->wstlen = wlen;
+					t1->wstptr[wlen] = L'\0';
+					t1->flags |= WSTRCUR;
+				} else
+					free_wstr(*lhs);
+#endif
 			} else {
 				size_t nlen = t1->stlen + t2->stlen;  
 				char *p;
