@@ -7,7 +7,7 @@
  */
 
 /*
- * Copyright (C) 1995 - 2001, 2003-2011 the Free Software Foundation, Inc.
+ * Copyright (C) 1995 - 2001, 2003-2012 the Free Software Foundation, Inc.
  * 
  * This file is part of GAWK, the GNU implementation of the
  * AWK Programming Language.
@@ -33,10 +33,6 @@
 
 #include <dlfcn.h>
 
-#ifdef __GNUC__
-static unsigned long long dummy;	/* fake out gcc for dynamic loading? */
-#endif
-
 /* do_ext --- load an extension */
 
 NODE *
@@ -54,12 +50,6 @@ do_ext(int nargs)
 	static short warned = FALSE;
 #endif
 
-#ifdef __GNUC__
-	AWKNUM junk;
-
-	junk = (AWKNUM) dummy;
-#endif
-
 	if (do_sandbox)
 		fatal(_("extensions are not allowed in sandbox mode"));
 
@@ -239,8 +229,15 @@ get_argument(int i)
 
 	t = GET_PARAM(i);
 
-	if (t->type == Node_array_ref)
+	if (t->type == Node_array_ref) {
+		if (t->orig_array->type == Node_var) {
+			/* already a scalar, can no longer use it as array */ 
+			t->type = Node_var;
+			t->var_value = Nnull_string;
+			return t;
+		}
 		return t->orig_array; 	/* Node_var_new or Node_var_array */
+	}
 	if (t->type == Node_var_new || t->type == Node_var_array)
 		return t;
 	return t->var_value;
